@@ -1,6 +1,8 @@
 """
 Shared pytest fixtures for unit tests.
 """
+from pathlib import Path
+
 import numpy as np
 import pytest
 from scipy.integrate import solve_ivp
@@ -12,7 +14,6 @@ from pysindy.feature_library import FourierLibrary
 from pysindy.feature_library import GeneralizedLibrary
 from pysindy.feature_library import PDELibrary
 from pysindy.feature_library import PolynomialLibrary
-from pysindy.feature_library import SINDyPILibrary
 from pysindy.utils.odes import logistic_map
 from pysindy.utils.odes import logistic_map_control
 from pysindy.utils.odes import logistic_map_multicontrol
@@ -20,6 +21,29 @@ from pysindy.utils.odes import lorenz
 from pysindy.utils.odes import lorenz_control
 
 
+def pytest_addoption(parser):
+    parser.addoption(
+        "--external-notebook",
+        action="append",
+        default=[],
+        help=(
+            "name of notebook to test.  Only valid if running"
+            " test_notebooks.test_external"
+        ),
+    )
+
+
+def pytest_generate_tests(metafunc):
+    if "external_notebook" in metafunc.fixturenames:
+        metafunc.parametrize(
+            "external_notebook",
+            [
+                Path(f.lstrip('"').rstrip('"'))
+                for f in metafunc.config.getoption("external_notebook")
+            ],
+        )
+
+
 @pytest.fixture
 def data_1d():
     t = np.linspace(0, 5, 100)
@@ -208,7 +232,7 @@ def data_2d_resolved_pde():
 @pytest.fixture
 def data_derivative_1d():
     x = 2 * np.linspace(1, 100, 100)
-    x_dot = 2 * np.ones(100).reshape(-1, 1)
+    x_dot = 2 * np.ones(100)
     return x, x_dot
 
 
@@ -216,7 +240,7 @@ def data_derivative_1d():
 def data_derivative_quasiperiodic_1d():
     t = np.arange(1000) * 2 * np.pi / 1000
     x = 2 * np.sin(t)
-    x_dot = 2 * np.cos(t).reshape(-1, 1)
+    x_dot = 2 * np.cos(t)
     return t, x, x_dot
 
 
@@ -333,20 +357,19 @@ def data_sindypi_library():
         lambda x: x**2,
         lambda x, y: x * y,
     ]
-    x_dot_library_functions = [lambda x: x]
     function_names = [
         lambda s: str(s),
         lambda s: str(s) + "^2",
         lambda s, t: str(s) + " " + str(t),
-        lambda s: str(s),
     ]
     t = np.linspace(0, 5, 500)
 
-    return SINDyPILibrary(
+    return PDELibrary(
         library_functions=library_functions,
-        x_dot_library_functions=x_dot_library_functions,
         function_names=function_names,
-        t=t,
+        temporal_grid=t,
+        implicit_terms=True,
+        derivative_order=1,
     )
 
 
@@ -371,7 +394,7 @@ def data_ode_library():
 
 @pytest.fixture
 def data_pde_library():
-    spatial_grid = np.linspace(0, 10)
+    spatial_grid = np.linspace(0, 10, 500)
     library_functions = [
         lambda x: x,
         lambda x: x**2,
